@@ -15,7 +15,7 @@ from xpra.platform.win32.common import (
     WNDCLASSEX, GetLastError,
     WNDPROC, LPCWSTR, LPWSTR, LPCSTR, DWORD,
     DefWindowProcW,
-    GetModuleHandleA, RegisterClassExW, UnregisterClassA,
+    GetModuleHandleA, RegisterClassExW, UnregisterClassW,
     CreateWindowExW, DestroyWindow,
     OpenClipboard, EmptyClipboard, CloseClipboard, GetClipboardData,
     GlobalLock, GlobalUnlock, GlobalAlloc, GlobalFree,
@@ -81,6 +81,8 @@ BLACKLISTED_CLIPBOARD_CLIENTS = [x for x in
                                  if x]
 log("BLACKLISTED_CLIPBOARD_CLIENTS=%s", BLACKLISTED_CLIPBOARD_CLIENTS)
 
+clipboard_window_class_name = "XpraWin32Clipboard"
+
 
 def is_blacklisted(owner_info):
     return any(owner_info.find(x)>=0 for x in BLACKLISTED_CLIPBOARD_CLIENTS)
@@ -120,13 +122,12 @@ class Win32Clipboard(ClipboardTimeoutHelper):
 
     def init_window(self):
         log("Win32Clipboard.init_window() creating clipboard window class and instance")
-        class_name = "XpraWin32Clipboard"
         self.wndclass = WNDCLASSEX()
         self.wndclass.cbSize = sizeof(WNDCLASSEX)
         self.wndclass.lpfnWndProc = WNDPROC(self.wnd_proc)
         self.wndclass.style =  win32con.CS_GLOBALCLASS
         self.wndclass.hInstance = GetModuleHandleA(0)
-        self.wndclass.lpszClassName = class_name
+        self.wndclass.lpszClassName = clipboard_window_class_name
         self.wndclass_handle = RegisterClassExW(byref(self.wndclass))
         log("RegisterClassExA(%s)=%#x", self.wndclass.lpszClassName, self.wndclass_handle)
         if self.wndclass_handle==0:
@@ -175,7 +176,7 @@ class Win32Clipboard(ClipboardTimeoutHelper):
         if wch:
             self.wndclass = None
             self.wndclass_handle = None
-            UnregisterClassA(wch, GetModuleHandleA(0))
+            UnregisterClassW(clipboard_window_class_name, GetModuleHandleA(0))
 
     def make_proxy(self, selection):
         proxy = Win32ClipboardProxy(self.window, selection,
