@@ -17,7 +17,8 @@ from ctypes.wintypes import (
 from ctypes.wintypes import (
     HANDLE, LPSTR, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ,
     LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA,
-    HINSTANCE, HMENU, ULONG, HHOOK,
+    HINSTANCE, HMENU, ULONG, HHOOK, LPMSG,
+    LCID, HKL,
     )
 #imported from this module but not used here:
 assert GetLastError
@@ -174,6 +175,9 @@ GetModuleHandleA = kernel32.GetModuleHandleA
 GetModuleHandleA.restype = HMODULE
 GetModuleHandleW = kernel32.GetModuleHandleW
 GetModuleHandleW.restype = HMODULE
+GetModuleHandleExA  = kernel32.GetModuleHandleExA
+GetModuleHandleExA.argtypes = [DWORD, LPCSTR, POINTER(HMODULE)]
+GetModuleHandleExA.restype = BOOL
 ConsoleCtrlHandler = WINFUNCTYPE(BOOL, DWORD)
 SetConsoleCtrlHandler = kernel32.SetConsoleCtrlHandler
 SetConsoleCtrlHandler.argtypes = (ConsoleCtrlHandler, BOOL)
@@ -225,6 +229,9 @@ QueryFullProcessImageNameA.argtypes = [HANDLE, DWORD, LPSTR, PDWORD]
 OpenProcess = kernel32.OpenProcess
 OpenProcess.restype = HANDLE
 OpenProcess.argtypes = [DWORD, BOOL, DWORD]
+GetThreadLocale = kernel32.GetThreadLocale
+GetThreadLocale.argtypes = []
+GetThreadLocale.restype = LCID
 
 user32 = WinDLL("user32", use_last_error=True)
 RegisterClassExA = user32.RegisterClassExA
@@ -290,12 +297,20 @@ GetMonitorInfoW.argtypes = [HMONITOR, POINTER(MONITORINFOEX)]
 GetMonitorInfoW.restype = BOOL
 UnhookWindowsHookEx = user32.UnhookWindowsHookEx
 CallNextHookEx = user32.CallNextHookEx
+CallNextHookEx.restype = LRESULT
+CallNextHookEx.argtyps = [HHOOK, INT, WPARAM, LPARAM]
 SetWindowsHookExA = user32.SetWindowsHookExA
 SetWindowsHookExA.restype = HHOOK
 SetWindowsHookExA.argtypes = [INT, HANDLE, HINSTANCE, DWORD]
 GetMessageA = user32.GetMessageA
+GetMessageA.restype = BOOL
+GetMessageA.argtypes = [LPMSG, HWND, UINT, UINT]
 TranslateMessage = user32.TranslateMessage
+TranslateMessage.restype = BOOL
+TranslateMessage.argtypes = [LPMSG]
 DispatchMessageA = user32.DispatchMessageA
+DispatchMessageA.restype = LRESULT
+DispatchMessageA.argtypes = [LPMSG]
 MapVirtualKeyW = user32.MapVirtualKeyW
 GetKeyboardState = user32.GetKeyboardState
 GetKeyboardState.argtypes = [PBYTE]
@@ -312,6 +327,8 @@ keybd_event = user32.keybd_event
 GetKeyState = user32.GetKeyState
 GetKeyState.restype = SHORT
 GetKeyboardLayout = user32.GetKeyboardLayout
+GetKeyboardLayout.argtypes = [DWORD]
+GetKeyboardLayout.restype = HKL
 GetKeyboardLayoutList = user32.GetKeyboardLayoutList
 GetKeyboardLayoutList.argtypes = [c_int, POINTER(HANDLE*32)]
 GetKeyboardLayoutName = user32.GetKeyboardLayoutNameA
